#import "MiscString.Levenstein.h"

@implementation MiscString(Levenstein)

#define	MAX_L	12
#define	RPL	1
#define	DEL	1
#define	INS	1
#define	SWP	1

#define	Mat(i)	(mat[i]&0x7fff)
#define	pSwp	0x8000
#define	msk(a)	(a&=0x7fff)
#define	isSwp(v)	((v)&pSwp)
#define	dVal(v)	((v)&0x7fff)

static int	min(int a, int b, int c)
{	msk(a),msk(b),msk(c);
	return a<b? (a<c? a: (b<c? b: c)): (b<c? b: (c<a? c: a));
}
/*
static int	minW(int a, int b, int c,short *which)
{	msk(a),msk(b),msk(c);
	return a<b? (a<c? *which=0,a: (b<c? *which=1,b: (*which=2,c))):
		(b<c? *which=1,b: (c<a? *which=2,c: (*which=0,a)));
}
*/

//cost2transf s1 into s2
#define	pIns	dist[-1]
/*
static int	distanceO2(const char *s1,const char *s2,short limit,short rpl,short ins,short del)
{	short	i,i1,i2,l1=strlen(s1),l2=strlen(s2),mat[l2+1],pRpl,pDel,*dist,minC;
	for (i=0; i<=l2; i++) mat[i]=i*ins;
	for (i1=0; i1<l1; i1++)
	{	for (i2=0,dist=mat+1,pRpl=mat[0],minC=mat[0]=pRpl+del; i2<l2; i2++,dist++,pRpl=pDel)
		{	pDel=*dist,*dist=min(pRpl+(s2[i2]==s1[i1]? 0: rpl),pDel+del,pIns+ins);
			if (*dist<minC) minC=*dist;
		} if (minC>limit) return minC;		//<*>minC is lower limit for dist.
	} return dist[-1];
}
*/

//	<N>will be correct and sensible only for swp<=2*rpl
static int	distanceO2S(const char *s1,const char *s2,short limit,short rpl,short ins,
	short del,short swp)
{	short	i,i1,i2,l1=strlen(s1),l2=strlen(s2),mat[l2+1],pRpl,pDel,*dist=0,minC,dSwp;
	for (i=0; i<=l2; i++) mat[i]=i*ins;

	for (i1=0; i1<l1; i1++)
	{	for (i2=0,dist=mat+1,pRpl=mat[0],minC=mat[0]=pRpl+del; i2<l2; i2++,dist++,pRpl=pDel)
		{	dSwp=0,pDel=*dist,*dist=min(pRpl+((isSwp(pRpl) || s2[i2]==s1[i1])? 0:
				(s2[i2+1]==s1[i1] && s2[i2]==s1[i1+1]? (dSwp=1,swp): rpl)),
					pDel+del,pIns+ins);
			if (dSwp) *dist|=pSwp;
			if (*dist<minC) minC=*dist;
		}
		if (minC>limit) return minC;		//<*>minC is lower limit for dist.
	}
	return dist[-1];
}

//	is s1 in s2
typedef struct {
	short	rpl,ins,del;
	int		testedLn,minLn;
} DistKtxt;
#define	Rpl	k->rpl
#define	Ins	k->ins
#define	Del	k->del
static int	distanceIn(const char *s1,const char *s2,int l1,int l2,short limit,DistKtxt *k)
{	int		i,i1,i2,mat[l1+1],pRpl,pDel,*dist,minC,mRet=limit+1,mLn=0;
	for (i=0; i<=l1; i++) mat[i]=i*Del;
	for (i2=0; i2<l2; i2++)
	{	for (i1=0,dist=mat+1,pRpl=mat[0],minC=mat[0]=pRpl+Ins; i1<l1; i1++,dist++,pRpl=pDel)
		{	pDel=*dist,*dist=min(pRpl+(s2[i2]==s1[i1]? 0: Rpl),pDel+Ins,pIns+Del);
			if (*dist<minC) minC=*dist;
		}
		if (minC>limit) return k->testedLn=i2+1,minC;//<*>! minC isLow Lim4dist.
		if (dist[-1]<=mRet) mRet=dist[-1],mLn=i2+1;
		if (dist[-1]<=minC) break;	//nothing to expect now
	} return k->testedLn=mLn,mRet;
}

static int	overlap(const char *s1,const char *s2,int minOvlp,int limit,DistKtxt *dk,
	int *pos1,int *pos2)
{	int		l1=strlen(s1),l2=strlen(s2),p1,p2,i,mx,dist=limit+1,fndDist=limit+1,nb;

	for (*pos1=*pos2=-1,i=-l1+minOvlp,mx=l2-minOvlp; i<=mx; i++)
	{	p1=i<0? -i: 0, p2=i<0? 0: i;
		if ((dist=distanceIn(s1+p1,s2+p2,l1-p1,l2-p2,limit,dk))<=limit)
		{	if (nb=dist<fndDist) fndDist=dist,*pos1=p1,*pos2=p2,dk->minLn=dk->testedLn;
			if (!dist || !nb) return fndDist;
		}
	}
	return dist;
}

-(unsigned) levensteinDistanceFor:(const char *) str cancelBeyond:(int) maxModifications
				   matchSubstring:(BOOL) ssFlag caseSensitive:(BOOL) caseFlag
{	int dummy;
	DistKtxt dk={RPL,INS,DEL};
	if(!caseFlag)
	{	int erg;
		MiscString *bufferStr=[self copy],*strStr=[[MiscString alloc] initString:str];
		[bufferStr toUpper];[strStr toUpper];
		erg=(  ssFlag ? overlap([strStr stringValue],[bufferStr stringValue],[strStr length],maxModifications,&dk,&dummy,&dummy):
					distanceO2S([strStr stringValue],[bufferStr stringValue],maxModifications, RPL,INS,DEL,SWP));
		[bufferStr free];[strStr free];
		return erg;
	} else
	{	return (ssFlag ? overlap(str,buffer,strlen(str),maxModifications,&dk,&dummy,&dummy):
					 distanceO2S(str,buffer,maxModifications, RPL,INS,DEL,SWP));
	}
}
@end
